<?php
/*
 *
 * PDF/X-5n starter:
 * Create PDF/X-5n conforming output with an image and DeviceN color
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9.1
 * Required data: font file, image file, 7-colorant ('xCLR') ICC output
 *                intent profile
 */

/* This is where the data files are. Adjust as necessary. */
$searchpath = dirname(__FILE__,3)."/input";

$imagefile = "zebra.tif";
// 7-colorant (xCLR) output intent profile.
$profilename = "2020_13.003_FOGRA55beta_CL_Profile.icc";

// Transform function for 2-color Lab-based spot color
$transform2 =
    "% DeviceN transform function for N=2 in CIE L*a*b* color space\n" .
    "% Lab color values of input colors must be listed here:\n"   .
    "64.0497  66.2296  85.5331        % color 1: PANTONE Orange 021 C\n" .
    "21.6373  57.9144 -71.8210        % color 2: PANTONE Violet C\n" .
    "% blend L values\n"                                          .
    "7 index 6 index mul              % t1*L1\n"                  .
    "7 index 4 index mul              % t2*L2\n"                  .
    "add\n"                                                       .
    "9 1 roll                         % bottom: L\n"              .
    "% blend a values\n"                                          .
    "7 index 5 index mul              % t1*a1\n"                  .
    "7 index 3 index mul              % t2*a2\n"                  .
    "add\n"                                                       .
    "9 1 roll                         % bottom: a\n"              .
    "% blend b values\n"                                          .
    "7 index 4 index mul              % t1*b1\n"                  .
    "7 index 2 index mul              % t2*b2\n"                  .
    "add\n"                                                       .
    "9 1 roll                         % bottom: b\n"              .
    "pop pop pop pop pop pop pop pop\n";

try {
    $p = new pdflib();

    /* check if PDFlib 9.1 or newer is installed */
    if (($p->get_option("major", "")*100)+($p->get_option("minor", "")*10) <910)
        throw new Exception("Error: PDFlib 9.1.0 or newer required, but " .
            "PDFlib " . $p->get_string(-1, "version") . " installed" );

    /*
     * Set errorpolicy to return, this means we must check return
     * values of load_font() etc.
     * Set the search path for fonts and images etc.
     */
    $p->set_option("errorpolicy=return SearchPath={{" . $searchpath . "}}");

    $p->set_option("stringformat=utf8");

    if ($p->begin_document("", "pdfx=PDF/X-5n") == 0) {
        throw new Exception("Error: " + $p->get_errmsg());
    }

    $p->set_info("Creator", "PDFlib starter sample");
    $p->set_info("Title", 'starter_pdfx5n');

    if ($p->load_iccprofile($profilename, "usage=outputintent") == 0) {
        print("Error: " . $p->get_errmsg(). "\n");
        print("An n-colorant ('xCLR') printer profile is required as output intent for PDF/X-5n.\n");
        exit(1);
    }

    $p->begin_page_ext(0, 0, "width=a4.width height=a4.height");

    /* Font embedding is required for PDF/X */
    $font = $p->load_font("NotoSerif-Regular", "unicode", "embedding");

    if ($font == 0) {
        throw new Exception("Error: " . $p->get_errmsg());
    }
    
    $p->fit_textline("PDF/X-5n starter sample with 7-color output intent",
                    50, 800, "fontsize=18 font=" . $font);

    /* Create a DeviceN color space with two of the device's process
     * colors.
     * The following types of spot colors can be used for PDF/X-5n:
     * - all built-in Pantone and HKS colors
     * - spot colors described in the n-colorant output intent profile
     * - custom spot colors defined with makespotcolor()
     */
    $devicen = $p->create_devicen(
                "names={{PANTONE Orange 021 C} {PANTONE Violet C}} alternate=lab transform={{"
                    . $transform2 . "}} errorpolicy=exception");
    $p->set_graphics_option("fillcolor={devicen " . $devicen . " 0.9 0.2}");

    /* Fill a circle with the DeviceN color defined above */
    $p->circle(400, 600, 100);
    $p->fill();

    /* The RGB image needs an ICC profile; we use sRGB. */
    $image = $p->load_image("auto", $imagefile, "iccprofile=sRGB");

    if ($image == 0) {
        throw new Exception("Error: " . $p->get_errmsg());
    }

    /* Place a diagonal stamp across the image area */
    $width = $p->info_image($image, "width", "");
    $height = $p->info_image($image, "height", "");

    /* Place the image on the page and close it */
    $p->fit_image($image, 0.0, 0.0, "");
    $p->close_image($image);

    $optlist = "fontsize=48 font=" . $font . " boxsize={" . $width . " "
                    . $height . "} stamp=ll2ur";
    $p->fit_textline("Zebra", 0, 0, $optlist);

    $p->end_page_ext("");

    $p->end_document("");

    $buf = $p->get_buffer();
    $len = strlen($buf);

    header("Content-type: application/pdf");
    header("Content-Length: $len");
    header("Content-Disposition: inline; filename=starter_pdfx5n.pdf");
    print $buf;
}
catch (PDFlibException $e) {
    echo("PDFlib exception occurred in starter_pdfx5n sample:" .
    "[" . $e->get_errnum() . "] " . $e->get_apiname()
            . ": " . $e->get_errmsg());
    exit(1);
}
catch (Throwable $e) {
    echo($e);
    exit(1);
}

$p = 0;

?>